/*
 * Created by Martin Sarkozi at 2011
 * as project for subject "Physical-based Animations and Mathematical Modeling"
 * 
 * project is demonstrating Sweep And Prune algorithm destined for
 * searching possible collisions of AABB boxes
 * 
 * implementation is not effective, but goal of this application is
 * to demonstrate the idea of this algorithm.
 * 
 * To compile project, openCV is needed
 * 
 */
#include <time.h>
#include <iostream>
#include <vector>

#include <opencv/cv.h>
#include <opencv/highgui.h>

#include "PM.h"
#include "physicalWorld.h"
#include "sweepAndPrune.h"
#include "bruteForce.h"

#define BILLION  1000000000L;

int main(int argc, char *argv[])
{
	float dt = 0.1;
	srand(time(NULL));
	struct timespec start1, stop1;
	struct timespec start2, stop2;
	
	physicalWorld *world = new physicalWorld(800, 600, dt);
	
	SweepAndPrune *SAP = new SweepAndPrune();
	SAP->world = world;
	world->SAP = SAP;
	
	BruteForce *BF = new BruteForce();
	BF->world = world;
	
	int box_num = (rand()%10) + 5;
	for(int i = 0; i < box_num; ++i) {
		int rndX  = rand()%400;
		int rndY  = rand()%400;
		int rndW  = rand()%100;
		int rndH  = rand()%100;
		int rnddX = (rand()%40) - 20;
		int rnddY = (rand()%40) - 20;
		world->addBox(rndX, rndY, rndW, rndH, rnddX, rnddY);
	}
	for(int i = 0; i < SAP->cornersX->size(); ++i)
	{
		cout << i << " " << SAP->cornersX->at(i)->owner->id << " " << *(SAP->cornersX->at(i)->value) << " " << SAP->cornersX->at(i)->is_start << endl;
	}
	
	PM *pmanagement = new PM(world->boxes->size());
	SAP->pmanagement = pmanagement;
	
	SAP->initialCheckIntersections();
	
	IplImage *output = cvCreateImage(cvSize(800, 600), 8, 3);
	
	CvFont font;
	cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, 1.,1.,0,1);
	stringstream s;
	CvScalar color;
	double accum1;
	double accum2;
	while(1)
	{
		cvSetZero(output);
		clock_gettime( CLOCK_REALTIME, &start1);
		int pm_colid = pmanagement->all_collisions();
		clock_gettime( CLOCK_REALTIME, &stop1);
		clock_gettime( CLOCK_REALTIME, &start2);
		int bf_colid = BF->all_collisions();
		clock_gettime( CLOCK_REALTIME, &stop2);
		accum1 = ( stop1.tv_sec - start1.tv_sec )
			+ (double)( stop1.tv_nsec - start1.tv_nsec )
			/ (double)BILLION;
		accum2 = ( stop2.tv_sec - start2.tv_sec )
			+ (double)( stop2.tv_nsec - start2.tv_nsec )
			/ (double)BILLION;
		cout << "SAP: " << accum1 << "; BF: " << accum2 << endl;
		for(int i = 0; i < world->boxes->size(); ++i)
		{
			if(pmanagement->is_colliding(world->boxes->at(i)))
				color = cvScalar(0, 0, 255);
			else if(pmanagement->is_semicolliding(world->boxes->at(i)))
				color = cvScalar(0, 128, 128);
			else
				color = cvScalar(0, 128, 0);
			cvRectangle(output, cvPoint(world->boxes->at(i)->xStart, world->boxes->at(i)->yStart),
				cvPoint(world->boxes->at(i)->xEnd, world->boxes->at(i)->yEnd), color
			);
			s.str("");
			s << world->boxes->at(i)->id;
			cvPutText(output, s.str().c_str(), cvPoint(world->boxes->at(i)->xStart, world->boxes->at(i)->yStart),
				&font, cvScalar(255, 255, 255)
			);
		}
		cvShowImage("output", output);
		world->step();
		if((cvWaitKey(1000.*dt) & 255) == 27) break;
	}
	return 0;
}
