#include "mainwindow.h"
#include "mainwindow_ui.h"
#include <time.h>

#define BILLION  1000000000L;

MainWindow::MainWindow ( QWidget* parent ): QMainWindow(parent), ui(new Ui::MainWindowClass)
{
	srand(time(NULL));
	float dt = 0.1;
	ui->setupUi(this);
	
	timer = new QTimer(this);
	
	sc = new QGraphicsScene(this);
	sc->setSceneRect(QRect(0, 0, 500, 500));
	ui->graphicsView->setScene(sc);
	QBrush brush(QColor(0, 0, 0, 255));
	brush.setStyle(Qt::SolidPattern);
	sc->setBackgroundBrush(brush);

	world = new physicalWorld(500, 500, dt);
	
	SAP = new SweepAndPrune();
	SAP->world = world;
	world->SAP = SAP;
	
	BF = new BruteForce();
	BF->world = world;
	
	int box_num = (rand()%10) + 5;
	for(int i = 0; i < box_num; ++i) {
		int rndX  = rand()%400;
		int rndY  = rand()%400;
		int rndW  = rand()%100;
		int rndH  = rand()%100;
		int rnddX = (rand()%40) - 20;
		int rnddY = (rand()%40) - 20;
		world->addBox(rndX, rndY, rndW, rndH, rnddX, rnddY);
	}
	
	pmanagement = new PM(world->boxes->size());
	SAP->pmanagement = pmanagement;
	
	SAP->initialCheckIntersections();
	
	boxes = new QGraphicsRectItem*[world->boxes->size()];
	QPen pen(QColor(255, 255, 255, 255));
	pen.setWidth(1);
	
	for(int i = 0; i < world->boxes->size(); ++i)
	{
//		boxes[i] = new QGraphicsRectItem(this);
		boxes[i] = sc->addRect(
			QRectF(world->boxes->at(i)->xStart, 
				   world->boxes->at(i)->yStart, 
				   world->boxes->at(i)->xEnd - world->boxes->at(i)->xStart,
				   world->boxes->at(i)->yEnd - world->boxes->at(i)->yStart
  				), 
			pen, brush);
	}
	ui->graphicsView->show();
	
	timer = new QTimer(this);
	timer->setInterval(1000*dt);
	
	connect(timer, SIGNAL(timeout()), this, SLOT(timeout()));

	timer->start();
	
}

void MainWindow::timeout()
{
	struct timespec start1, stop1;
	struct timespec start2, stop2;
	double accum1, accum2;
	world->step();
	clock_gettime( CLOCK_REALTIME, &start1);
	int pm_colid = pmanagement->all_collisions();
	clock_gettime( CLOCK_REALTIME, &stop1);
	clock_gettime( CLOCK_REALTIME, &start2);
	int bf_colid = BF->all_collisions();
	clock_gettime( CLOCK_REALTIME, &stop2);
	accum1 = ( stop1.tv_sec - start1.tv_sec )
		+ (double)( stop1.tv_nsec - start1.tv_nsec )
		/ (double)BILLION;
	accum2 = ( stop2.tv_sec - start2.tv_sec )
		+ (double)( stop2.tv_nsec - start2.tv_nsec )
		/ (double)BILLION;

	for(int i = 0; i < world->boxes->size(); ++i)
	{
		if(pmanagement->is_colliding(world->boxes->at(i)))
		{
			QPen pen(QColor(255, 0, 0, 255));
			pen.setWidth(1);
			boxes[i]->setPen(pen);
		}
		else if(pmanagement->is_semicolliding(world->boxes->at(i)))
		{
			QPen pen(QColor(128, 128, 0, 255));
			pen.setWidth(1);
			boxes[i]->setPen(pen);
		}
		else
		{
			QPen pen(QColor(0, 128, 0, 255));
			pen.setWidth(1);
			boxes[i]->setPen(pen);
		}
		QRectF rrect(world->boxes->at(i)->xStart, 
					 world->boxes->at(i)->yStart, 
					 world->boxes->at(i)->xEnd - world->boxes->at(i)->xStart,
					 world->boxes->at(i)->yEnd - world->boxes->at(i)->yStart
					);
		boxes[i]->setRect(rrect);
	}
	
	ui->label->setText(QString("SAP: ") + QString::number(accum1));
	ui->label_2->setText(QString("BF: ") + QString::number(accum2));
}
