#include <vector>

#include "sweepAndPrune.h"

SweepAndPrune::SweepAndPrune()
{
	this->cornersX = new std::vector<boxCorner *>(0);
	this->cornersY = new std::vector<boxCorner *>(0);
}

/*
 * add corners of new box to ordered list
 */
void SweepAndPrune::updateListByAdding(AABB* box)
{
	if(this->cornersX->size() == 0) // set of first box(sorting not required)
	{
		this->cornersX->push_back(new boxCorner(box, &(box->xStart), true));
		this->cornersX->push_back(new boxCorner(box, &(box->xEnd), false));
		this->cornersY->push_back(new boxCorner(box, &(box->yStart), true));
		this->cornersY->push_back(new boxCorner(box, &(box->yEnd), false));
	}
	else
	{
		this->cornersX->resize(this->cornersX->size()+2);
		bool first = true;
		int blank1 = this->cornersX->size()-1;
		int blank2 = this->cornersX->size()-2;
		for(int iter = this->cornersX->size()-3; iter >= 0; --iter)
		{
			if(first)
			{
				if(*(this->cornersX->at(iter)->value) > box->xEnd)
				{
					this->cornersX->at(iter+2) = this->cornersX->at(iter);
					blank1--;
					blank2--;
				}
				else
				{
					this->cornersX->at(iter+2) = new boxCorner(box, &(box->xEnd), false);
					blank1 = -1;
					first = false;
					iter++;
				}
			}
			else
			{
				if(*(this->cornersX->at(iter)->value) > box->xStart)
				{
					this->cornersX->at(iter+1) = this->cornersX->at(iter);
					blank2--;
				}
				else
				{
					this->cornersX->at(iter+1) = new boxCorner(box, &(box->xStart), true);
					blank2 = -1;
					break;
				}
			}
		}
		if(blank1 == 1) this->cornersX->at(1) = new boxCorner(box, &(box->xEnd), false);
		if(blank2 == 0) this->cornersX->at(0) = new boxCorner(box, &(box->xStart), true);

		this->cornersY->resize(this->cornersY->size()+2);
		first = true;
		blank1 = this->cornersY->size()-1;
		blank2 = this->cornersY->size()-2;
		for(int iter = this->cornersY->size()-3; iter >= 0; --iter)
		{
			if(first)
			{
				if(*(this->cornersY->at(iter)->value) > box->yEnd)
				{
					this->cornersY->at(iter+2) = this->cornersY->at(iter);
					blank1--;
					blank2--;
				}
				else
				{
					this->cornersY->at(iter+2) = new boxCorner(box, &(box->yEnd), false);
					blank1 = -1;
					first = false;
					iter++;
				}
			}
			else
			{
				if(*(this->cornersY->at(iter)->value) > box->yStart)
				{
					this->cornersY->at(iter+1) = this->cornersY->at(iter);
					blank2--;
				}
				else
				{
					this->cornersY->at(iter+1) = new boxCorner(box, &(box->yStart), true);
					blank2 = -1;
					break;
				}
			}
		}
		if(blank1 == 1) this->cornersY->at(1) = new boxCorner(box, &(box->yEnd), false);
		if(blank2 == 0) this->cornersY->at(0) = new boxCorner(box, &(box->yStart), true);
	}
}

/*
 * erase existing collisions with this box and erase box from corners list
 */
void SweepAndPrune::updateListByRemoving (AABB* box)
{
	for(unsigned int i = 0; i < this->cornersX->size(); ++i)
	{
		this->pmanagement->resetXCollision(box, this->cornersX->at(i)->owner);
	}
	for(unsigned int i = 0; i < this->cornersY->size(); ++i)
	{
		this->pmanagement->resetYCollision(box, this->cornersY->at(i)->owner);
	}
	for(unsigned int i = 0; i < this->cornersX->size(); ++i)
	{
		if(this->cornersX->at(i)->owner->id == box->id)
		{
			this->cornersX->erase(this->cornersX->begin()+i);
		}
	}
	for(unsigned int i = 0; i < this->cornersY->size(); ++i)
	{
		if(this->cornersY->at(i)->owner->id == box->id)
		{
			this->cornersY->erase(this->cornersY->begin()+i);
		}
	}
}


/*
 * set collisions of created objects when application start
 */
void SweepAndPrune::initialCheckIntersections()
{
	std::vector<AABB *> *openedBoxes = new std::vector<AABB *>();

	for(unsigned int i = 0; i < this->cornersX->size(); ++i)
	{
		boxCorner *tmp1 = this->cornersX->at(i);
		if(tmp1->is_start) // found starting corner of box
		{
			openedBoxes->push_back(tmp1->owner);
			// in sorted list of corners, each corner between two marginal corners of AABB box is potential collision on axis
			for(unsigned int j = 0; j < openedBoxes->size()-1; ++j)
			{
				this->pmanagement->setXCollision(tmp1->owner, openedBoxes->at(j));
			}
		}
		else // founding ending corner of box
		{
			for(int j = 0; j < openedBoxes->size(); ++j)
			{
				if(openedBoxes->at(j)->id == tmp1->owner->id)
				{
					openedBoxes->erase(openedBoxes->begin()+j);
				}
			}
		}
	}
	// the same for Y axis
	for(unsigned int i = 0; i < this->cornersY->size(); ++i)
	{
		boxCorner *tmp1 = this->cornersY->at(i);
		if(tmp1->is_start)
		{
			openedBoxes->push_back(tmp1->owner);
			for(unsigned int j = 0; j < openedBoxes->size()-1; ++j)
			{
				this->pmanagement->setYCollision(tmp1->owner, openedBoxes->at(j));
			}
		}
		else
		{
			for(unsigned int j = 0; j < openedBoxes->size(); ++j)
			{
				if(openedBoxes->at(j)->id == tmp1->owner->id)
				{
					openedBoxes->erase(openedBoxes->begin()+j);
				}
			}
		}
	}
}

void SweepAndPrune::updateObject(AABB* obj, float nx, float ny)
{
	for(unsigned int i = 0; i < this->cornersX->size(); ++i)
	{
		if(this->cornersX->at(i)->owner->id == obj->id)
		{
			if(this->cornersX->at(i)->is_start)
			{ // left corner
				if(nx < 0)
				{ // move left
					if(i == 0) continue; // nothig to test
					if(*(this->cornersX->at(i-1)->value) > *(this->cornersX->at(i)->value))
					{ // switch corners
						if(!this->cornersX->at(i-1)->is_start)
						{ // add pair
							this->pmanagement->setXCollision(this->cornersX->at(i)->owner, this->cornersX->at(i-1)->owner);
						}
						boxCorner *tmp = this->cornersX->at(i-1);
						this->cornersX->at(i-1) = this->cornersX->at(i);
						this->cornersX->at(i) = tmp;
					}
				}
				else
				{ // move right
					if(i == this->cornersX->size()-1) continue; // nothig to test
					if(*(this->cornersX->at(i+1)->value) < *(this->cornersX->at(i)->value))
					{ // switch corners
						if(!this->cornersX->at(i+1)->is_start)
						{ // remove pair
							this->pmanagement->resetXCollision(this->cornersX->at(i)->owner, this->cornersX->at(i+1)->owner);
						}
						boxCorner *tmp = this->cornersX->at(i+1);
						this->cornersX->at(i+1) = this->cornersX->at(i);
						this->cornersX->at(i) = tmp;
					}
				}
			}
			else
			{ // right corner
				if(nx < 0)
				{ // move left
					if(i == 0) continue; // nothig to test
					if(*(this->cornersX->at(i-1)->value) > *(this->cornersX->at(i)->value))
					{ // switch corners
						if(this->cornersX->at(i-1)->is_start)
						{ // remove pair
							this->pmanagement->resetXCollision(this->cornersX->at(i)->owner, this->cornersX->at(i-1)->owner);
						}
						boxCorner *tmp = this->cornersX->at(i-1);
						this->cornersX->at(i-1) = this->cornersX->at(i);
						this->cornersX->at(i) = tmp;
					}
				}
				else
				{ // move right
					if(i == this->cornersX->size()-1) continue; // nothig to test
					if(*(this->cornersX->at(i+1)->value) < *(this->cornersX->at(i)->value))
					{ // switch corners
						if(this->cornersX->at(i+1)->is_start)
						{ // add pair
							this->pmanagement->setXCollision(this->cornersX->at(i)->owner, this->cornersX->at(i+1)->owner);
						}
						boxCorner *tmp = this->cornersX->at(i+1);
						this->cornersX->at(i+1) = this->cornersX->at(i);
						this->cornersX->at(i) = tmp;
					}
				}
				break; // after successful update right corner, all is done for X axis
			}
		}
	}
	for(unsigned int i = 0; i < this->cornersY->size(); ++i)
	{
		if(this->cornersY->at(i)->owner->id == obj->id)
		{
			if(this->cornersY->at(i)->is_start)
			{ // upper corner
				if(ny < 0)
				{ // move up
					if(i == 0) continue; // nothig to test
					if(*(this->cornersY->at(i-1)->value) > *(this->cornersY->at(i)->value))
					{
						if(!this->cornersY->at(i-1)->is_start)
						{ // add pair
							this->pmanagement->setYCollision(this->cornersY->at(i)->owner, this->cornersY->at(i-1)->owner);
						}
						boxCorner *tmp = this->cornersY->at(i-1);
						this->cornersY->at(i-1) = this->cornersY->at(i);
						this->cornersY->at(i) = tmp;
					}
				}
				else
				{ // move down
					if(i == this->cornersY->size()-1) continue; // nothig to test
					if(*(this->cornersY->at(i+1)->value) < *(this->cornersY->at(i)->value))
					{
						if(!this->cornersY->at(i+1)->is_start)
						{ // remove pair
							this->pmanagement->resetYCollision(this->cornersY->at(i)->owner, this->cornersY->at(i+1)->owner);
						}
						boxCorner *tmp = this->cornersY->at(i+1);
						this->cornersY->at(i+1) = this->cornersY->at(i);
						this->cornersY->at(i) = tmp;
					}
				}
			}
			else
			{ // lower corner
				if(ny < 0)
				{ // move up
					if(i == 0) continue; // nothig to test
					if(*(this->cornersY->at(i-1)->value) > *(this->cornersY->at(i)->value))
					{
						if(this->cornersY->at(i-1)->is_start)
						{ // remove pair
							this->pmanagement->resetYCollision(this->cornersY->at(i)->owner, this->cornersY->at(i-1)->owner);
						}
						boxCorner *tmp = this->cornersY->at(i-1);
						this->cornersY->at(i-1) = this->cornersY->at(i);
						this->cornersY->at(i) = tmp;
					}
				}
				else
				{ // move down
					if(i == this->cornersY->size()-1) continue; // nothig to test
					if(*(this->cornersY->at(i+1)->value) < *(this->cornersY->at(i)->value))
					{
						if(this->cornersY->at(i+1)->is_start)
						{ // add pair
							this->pmanagement->setYCollision(this->cornersY->at(i)->owner, this->cornersY->at(i+1)->owner);
						}
						boxCorner *tmp = this->cornersY->at(i+1);
						this->cornersY->at(i+1) = this->cornersY->at(i);
						this->cornersY->at(i) = tmp;
					}
				}
				break; // after successful update right corner, all is done for X axis
			}
		}
	}
}

